/*
 * Copyright 2016 (c) Anna Schumaker.
 */
#include <core/idle.h>
#include <core/settings.h>
#include <core/string.h>
#include <gui/builder.h>
#include <gui/filter.h>
#include <gui/model.h>
#include <gui/playlist.h>
#include <gui/sidebar.h>

static void __playlist_update_sizes(struct queue *);

void __playlist_row_activated(GtkTreeView *treeview, GtkTreePath *path,
			      GtkTreeViewColumn *col, gpointer data)
{
	gui_sidebar_filter_path_select(path);
	__playlist_update_sizes(&playlist_cur()->pl_queue);
}

static bool __playlist_queue_set_size(struct queue *queue, GtkTreeIter *iter)
{
	struct playlist *playlist = queue->q_private;
	bool   match = (gui_sidebar_iter_type(iter) == playlist->pl_type);
	gchar *name  = gui_sidebar_iter_name(iter);

	if (match)
		match = string_match(name, playlist->pl_name);
	if (match)
		gui_sidebar_iter_update(iter);
	g_free(name);

	return match;
}

static void __playlist_update_sizes(struct queue *queue)
{
	struct playlist *playlist = queue->q_private;
	GtkTreeModelFilter *filter;
	GtkTreeIter  parent, iter;

	if (playlist->pl_type == PL_LIBRARY) {
		gui_pl_library_update(playlist);
		return;
	} else if (playlist->pl_type == PL_ARTIST) {
		gui_pl_artist_update(playlist);
		return;
	} else if (playlist->pl_type == PL_USER) {
		gui_pl_user_update(playlist);
		return;
	} else if (playlist->pl_type == PL_SYSTEM) {
		gui_pl_system_update(playlist);
		return;
	}

	if (!gui_sidebar_iter_first(&parent))
		return;

	do {
		if (__playlist_queue_set_size(queue, &parent))
			goto out;
		if (gtk_tree_model_iter_children(gui_sidebar_model(), &iter, &parent)) {
			do {
				if (__playlist_queue_set_size(queue, &iter))
					goto out;
			} while (gui_sidebar_iter_next(&iter));
		}
	} while (gui_sidebar_iter_next(&parent));

out:
	filter = gui_sidebar_filter();
	gtk_tree_model_filter_refilter(filter);
}

static void *__playlist_init(struct queue *queue, void *data)
{
	struct playlist *playlist = (struct playlist *)data;

	if (playlist->pl_type == PL_ARTIST)
		gui_pl_artist_add(playlist);
	return playlist;
}

static void __playlist_deinit(struct queue *queue)
{
	gui_filter_clear_search(queue->q_private);
}

static void __playlist_added(struct queue *queue, unsigned int row)
{
	gui_model_add(queue->q_private, row);
	__playlist_update_sizes(queue);
}

static void __playlist_removed(struct queue *queue, unsigned int row)
{
	gui_model_remove(queue->q_private, row);
	__playlist_update_sizes(queue);
}

static void __playlist_cleared(struct queue *queue, unsigned int n)
{
	gui_model_clear(queue->q_private, n);
	__playlist_update_sizes(queue);
}

static void __playlist_updated(struct queue *queue, unsigned int n)
{
	gui_model_update(queue->q_private, n);
}

static bool __playlist_erase(struct queue *queue, struct track *track)
{
	struct playlist *playlist = queue->q_private;
	enum playlist_type_t type = playlist->pl_type;
	const gchar         *name = playlist->pl_name;

	switch (type) {
	case PL_SYSTEM:
		if (string_match(name, "Collection"))
			name = "Hidden";
		else if (!string_match(name, "Favorites") &&
			 !string_match(name, "Hidden")    &&
			 !string_match(name, "Queued Tracks"))
			break;
	case PL_USER:
		playlist_remove(type, name, track);
	default:
		break;
	};

	return false;
}

bool __gui_playlist_init_idle()
{
	GtkTreeSelection *selection;
	GtkTreeModel *filter;
	GtkTreeIter iter;

	filter    = GTK_TREE_MODEL(gui_sidebar_filter());
	selection = gtk_tree_view_get_selection(gui_sidebar_treeview());
	gtk_tree_model_get_iter_first(filter, &iter);
	gtk_tree_selection_select_iter(selection, &iter);

	return true;
}

void gui_playlist_init()
{
	gui_pl_system_init();
	gui_pl_artist_init();
	gui_pl_user_init();
	gui_pl_library_init();

	idle_schedule(IDLE_SYNC, __gui_playlist_init_idle, NULL);
}

const gchar *gui_playlist_cur()
{
	if (gui_model_get_playlist())
		return gui_model_get_playlist()->pl_name;
	return NULL;
}

struct queue_ops playlist_ops = {
	.qop_init    = __playlist_init,
	.qop_deinit  = __playlist_deinit,
	.qop_added   = __playlist_added,
	.qop_erase   = __playlist_erase,
	.qop_removed = __playlist_removed,
	.qop_cleared = __playlist_cleared,
	.qop_updated = __playlist_updated,
};
